//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
#property indicator_separate_window
#property indicator_buffers 2

//---- inputs
extern int     UniqueNum            = 13579; // change this number if you want to use multiple instances of this indicator
extern int     BarsToCount          = 500;   // restrict indicator to show data from only given number of bars
extern string  Separator_1 = "- - - - - - - - - - - - - - - - - - - - - - - - - - - - -";
extern int     BoxCalculationMethod = 2;     // >1< - static, >...< - ADR
extern int     BoxSize              = 10;    // >1< ; manual/static 
extern int     BarsToCalculateADR   = 100;   // >2< ; number of bars for dynamic box size calculation
extern int     ADR_divisor          = 2;     // if >2< is selected this number is used to calculate dynamic BoxSize
extern int     Reversal             = 3;     
extern int     HL_CC                = 1;     // price used for calculation  >1< - High/Low ; >...< - Close/Close
extern string  Separator_3 = "- - - - - - - - - - - - - - - - - - - - - - - - - - - - -";
extern int     ColumnWidth          = 1;
extern int     LastColumnShift      = 20;    // shifting to the left factor
extern string  Separator_4 = "- - - - - - - - - - - - - - - - - - - - - - - - - - - - -";
extern color   UpColor              = Blue;
extern color   DownColor            = Red;
extern color   BorderColor          = Aqua;
extern string  Separator_5 = "- - - - - - - - - - - - - - - - - - - - - - - - - - - - -";
extern bool    ShowPrice            = True;  // display CLOSE of current candle as label
extern bool    PriceAsLine          = True;  // >True< - show PRICE as line, >False< - show PRICE as label
extern color   PriceColor           = White;
extern string  Separator_6 = "- - - - - - - - - - - - - - - - - - - - - - - - - - - - -";
extern bool    ShowGrid             = False;
extern color   GridColor            = Black;


//---- buffers for scaling indicator window 
double   ind_Max[], ind_Min[];

//---- buffers used for calculation
double   start_price[], last_price[];
double   pr_h, pr_l, pbox;
int      index; 
int      ColNum = 0;

string   XO_direction;
string   short_name;
int      tmpWindow;

double   GridRefPoint = 0;
int      AVGBoxSize;
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
void init()
{
   InitializeBuffers();
   
   pbox = BoxSize*Point;
   
   return;
}
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
void deinit()
{
   CleanScreen();
   
   return;
}
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
void start()
{
   int countedBars = IndicatorCounted();
   
   if (countedBars<0) return;
   else { 
      CleanScreen(); Preparation(); 
   }
   
   AVGBoxSize = MathRound( CalculateAVG(BarsToCalculateADR)/ADR_divisor );
   
   if ( BoxCalculationMethod == 2 ) {
      pbox = AVGBoxSize*Point;  // dynamic Box
   } else {
      pbox = BoxSize*Point;     // static Box
   }
   
   for (index = Bars-1; index >= 0; index--) {
   
   if (BarsToCount == 0 || (BarsToCount > 0 && index <= BarsToCount)) {
      
      // calculation method - High/Low or Close/Close
      if (HL_CC == 1) {
         pr_h = MathFloor(High[index]/pbox)*pbox;
         pr_l = MathCeil (Low[index] /pbox)*pbox;
      } else {
         pr_h = MathFloor(Close[index]/pbox)*pbox;
         pr_l = MathCeil (Close[index]/pbox)*pbox;
      }
      
      // first box calculation
      if (ArrayRange(start_price,0) == 0) {
         ColNum = ResizeArray();
         if (Close[Bars-1]<Close[index]) {
            start_price[ColNum] = pr_l + pbox;
            last_price[ColNum] = pr_l;
            XO_direction = "DownBox active";
            GridRefPoint = pr_l + pbox;
            continue;
         } else {
            start_price[ColNum] = pr_h + pbox;
            last_price[ColNum] = pr_h;
            XO_direction = "UpBox active";
            GridRefPoint = pr_h + pbox;
            continue;
         }
      }
      
      if (XO_direction == "DownBox active") {
         if (pr_l <= last_price[ColNum] - pbox) {
            last_price[ColNum] = pr_l;
            continue;
         }
         if (pr_h >= last_price[ColNum] + pbox*Reversal) {
            ColNum = ResizeArray();
            start_price[ColNum] = last_price[ColNum-1] + pbox;
            last_price[ColNum] = pr_h;
            XO_direction = "UpBox active";
            continue;
         }
      } // "DownBox active"
    
      if (XO_direction == "UpBox active") {
         if (pr_h >= last_price[ColNum] + pbox) {
            last_price[ColNum] = pr_h;
            continue;
         }
         if (pr_l <= last_price[ColNum] - pbox*Reversal) {
            ColNum = ResizeArray();
            start_price[ColNum] = last_price[ColNum-1] - pbox;
            last_price[ColNum] = pr_l;
            XO_direction = "DownBox active";
            continue;
         }
      } // "UpBox active"
   } // BarsToCount
   } // index
   
   Set_Array_As_Series(true);
   
   if ( ShowPrice ) 
      ShowCurrentPrice();
   
   if ( ShowGrid )
      DrawGrid(NormalizeDouble(GridRefPoint,Digits), ColNum);
   
   ShowInfo();
   
   for ( ; ColNum >= 0; ColNum--) 
      CreateObject(ColNum);
   
   return;
}
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
void CreateObject(int tmpIdx) {
   
   static color tmpColor;
   int i = tmpIdx + LastColumnShift; // shifting last column
   int t = i*ColumnWidth - ColumnWidth +1;
   double prh, prl;
   
   string name = StringConcatenate( UniqueNum, " ", tmpWindow, " ", "X-O ", i );
   
   tmpWindow = WindowFind(short_name);
   
   if (t+ColumnWidth*2 > Bars)
      return;
   
   if (start_price[tmpIdx] == last_price[tmpIdx]) { 
      if (tmpColor == DownColor) tmpColor = UpColor;
      else tmpColor = DownColor;
   } else {
      if (start_price[tmpIdx] > last_price[tmpIdx]) tmpColor = DownColor; 
      else tmpColor = UpColor;
   }
   
   pr_h = NormalizeDouble( MathMax(start_price[tmpIdx], last_price[tmpIdx]), Digits);
   pr_l = NormalizeDouble( MathMin(start_price[tmpIdx], last_price[tmpIdx]), Digits);
   
   prh = NormalizeDouble(pr_h + pbox/2, Digits+1);
   prl = NormalizeDouble(pr_l - pbox/2, Digits+1);
   
   ObjectCreate(name,OBJ_RECTANGLE, tmpWindow ,Time[t],prh,Time[t+ColumnWidth],prl);
   ObjectSet(name,OBJPROP_COLOR,tmpColor);
   
   // correctly scale indicator window 
   ind_Max[t+ColumnWidth] = prh + pbox;
   ind_Min[t+ColumnWidth] = prl - pbox;
   
   for ( ; pr_l <= pr_h; pr_l = NormalizeDouble(pr_l + pbox, Digits) ) {
      
      prh = NormalizeDouble(pr_l + pbox/2, Digits+1); 
      prl = NormalizeDouble(pr_l - pbox/2, Digits+1);
      
      ObjectCreate(name+" "+DoubleToStr(pr_l,Digits),OBJ_RECTANGLE, tmpWindow ,Time[t],prl,Time[t+ColumnWidth],prh);
      ObjectSet(name+" "+DoubleToStr(pr_l,Digits), OBJPROP_COLOR, BorderColor);
      ObjectSet(name+" "+DoubleToStr(pr_l,Digits), OBJPROP_BACK, false);
   }
}
//+------------------------------------------------------------------+
void Preparation() { 
   
   ArrayResize(start_price,0);
   ArrayResize(last_price,0);
  
   Set_Array_As_Series (false);
}
//+------------------------------------------------------------------+
int ResizeArray() {
   
   ArrayResize(start_price,ArrayRange(start_price,0)+1);
   ArrayResize(last_price,ArrayRange(last_price,0)+1);
   
   return (ArrayRange(start_price,0)-1);
}
//+------------------------------------------------------------------+
void Set_Array_As_Series(bool x) {
   
   ArraySetAsSeries(start_price,x);
   ArraySetAsSeries(last_price,x);
}
//+------------------------------------------------------------------+
void ShowCurrentPrice() {
   
   tmpWindow = WindowFind(short_name);
   
   if (!PriceAsLine) {
      ObjectDelete(UniqueNum+ " " +tmpWindow+ " CurrPrice");
      ObjectCreate(UniqueNum+ " " +tmpWindow+ " CurrPrice", OBJ_ARROW, tmpWindow, Time[0+LastColumnShift]+3*Period()*60, Close[0] );
      ObjectSet(UniqueNum+ " " +tmpWindow+ " CurrPrice", OBJPROP_COLOR, PriceColor);
      ObjectSet(UniqueNum+ " " +tmpWindow+ " CurrPrice", OBJPROP_ARROWCODE, SYMBOL_RIGHTPRICE);
   } else {
      ObjectDelete(UniqueNum+ " " +tmpWindow+ " CurrPrice");
      ObjectCreate(UniqueNum+ " " +tmpWindow+ " CurrPrice", OBJ_HLINE, tmpWindow, 0, Close[0]);
      ObjectSet(UniqueNum+ " " +tmpWindow+ " CurrPrice", OBJPROP_COLOR, PriceColor);
      ObjectSet(UniqueNum+ " " +tmpWindow+ " CurrPrice", OBJPROP_WIDTH, 1);
      ObjectSet(UniqueNum+ " " +tmpWindow+ " CurrPrice", OBJPROP_STYLE, STYLE_DOT);
   }
}
//+------------------------------------------------------------------+ 
void CleanScreen() {
   
   tmpWindow = WindowFind(short_name);
   
   for (int k=ObjectsTotal()-1;k>=0;k--) {
      if (StringFind(ObjectName(k),UniqueNum+ " " +tmpWindow) >= 0)
            ObjectDelete(ObjectName(k));
   }
   
   Comment("");
}
//+------------------------------------------------------------------+ 
void InitializeBuffers() {
   
   SetIndexStyle(0,DRAW_LINE,EMPTY,EMPTY,clrNONE);
   SetIndexLabel(0,"X-O_max");
   SetIndexBuffer(0,ind_Max);
   
   SetIndexStyle(1,DRAW_LINE,EMPTY,EMPTY,clrNONE);
   SetIndexLabel(1,"X-O_min");
   SetIndexBuffer(1,ind_Min);
   
   if (BoxSize <= 0) BoxSize = 1;
   if (Reversal <= 1) Reversal = 2;
   if (BarsToCalculateADR<=0 || BarsToCalculateADR>=(Bars-1)) BarsToCalculateADR=1;
   if (ADR_divisor<=0) ADR_divisor=1;
   if (ColumnWidth <= 0 || ColumnWidth >= (Bars-1)) ColumnWidth = 1;
   if (LastColumnShift <=0 || LastColumnShift>=(Bars-1)) LastColumnShift=1;
   
   if ( BoxCalculationMethod == 1 ) 
      short_name = "  X-O (" +BoxSize+ " x " +Reversal+ ") Id(.:" +UniqueNum+ ":.)";
   else 
      short_name = "  X-O (dynamic" + " x " +Reversal+ ") Id(.:" +UniqueNum+ ":.)";
   
   IndicatorShortName(short_name);
}
//+------------------------------------------------------------------+ 
void DrawGrid (double tmpGridRefPoint, int tmpNbCol) {
   
   tmpWindow = WindowFind(short_name);
   
   string nameH = StringConcatenate( UniqueNum, " ", tmpWindow, " ", "GridH ");
   string nameV = StringConcatenate( UniqueNum, " ", tmpWindow, " ", "GridV ");
   
   double MaxInd = High[iHighest(NULL,0,MODE_HIGH,BarsToCount,0)];
   double MinInd = Low [iLowest(NULL,0,MODE_LOW,BarsToCount,0)];
   
   if (tmpGridRefPoint > MinInd) {
      int GridStartPoint = MathRound( (tmpGridRefPoint-MinInd)/pbox ) + 1;
      tmpGridRefPoint = NormalizeDouble(GridRefPoint - GridStartPoint*pbox, Digits);
   }
   
   int NbRow = MathCeil( (MaxInd-MinInd) / pbox );
   
   for (int l = NbRow+6; l >= 1; l--) {
      ObjectDelete(nameH +l);
      ObjectCreate(nameH +l, OBJ_TREND, tmpWindow, 
         Time[iBarShift(NULL,0,iTime(NULL,0,(tmpNbCol+LastColumnShift)*ColumnWidth+LastColumnShift/2))], 
         tmpGridRefPoint + pbox/2 + (l-2)*pbox, 
         Time[0]+(LastColumnShift/2)*Period()*60,
         tmpGridRefPoint + pbox/2 + (l-2)*pbox);
      ObjectSet(nameH +l, OBJPROP_COLOR, GridColor);
      ObjectSet(nameH +l, OBJPROP_STYLE, STYLE_DOT);
      ObjectSet(nameH +l, OBJPROP_RAY, false);
      ObjectSet(nameH +l, OBJPROP_SCALE, 0);
      ObjectSet(nameH +l, OBJPROP_BACK, true);
   }
   
   for (int n = tmpNbCol+LastColumnShift; n >= 0; n--) {
      ObjectDelete(nameV +n);
      ObjectCreate(nameV +n, OBJ_VLINE, tmpWindow, 
         Time[iBarShift(NULL,0,iTime(NULL,0,n*ColumnWidth+1))], 0);
      ObjectSet(nameV +n, OBJPROP_COLOR, GridColor);
      ObjectSet(nameV +n, OBJPROP_STYLE, STYLE_DOT);
      ObjectSet(nameV +n, OBJPROP_BACK, true);
   }
}
//+------------------------------------------------------------------+
void ShowInfo() {
   int startX = 10, startY = 5;
   int tmpLineLength = 20;
   int FontSize = 10;
   double tmpH = High[iHighest(NULL,0,MODE_HIGH,200,0)];
   double tmpL = Low[iLowest(NULL,0,MODE_LOW,200,0)];
   double mid = tmpH - (tmpH-tmpL);
   
   double tmpX,tmpY;
   
   tmpWindow = WindowFind(short_name);
   
   tmpX = startX + 2*FontSize;
   
   if ( BoxCalculationMethod == 1 )
      tmpY = startY*2 + 2*FontSize + 5;
   else 
      tmpY = startY*2 + 4*FontSize + 5;
   
   string nameBOX = StringConcatenate( UniqueNum, " ", tmpWindow, " ", "BOXinfo ");
   string nameAVG = StringConcatenate( UniqueNum, " ", tmpWindow, " ", "AVGinfo ");
   string nameSel = StringConcatenate(UniqueNum," ",tmpWindow," ","BoxActive ");
   string nameMeth = StringConcatenate(UniqueNum," ",tmpWindow," ","Method ");
   string nameSep1 = StringConcatenate( UniqueNum, " ", tmpWindow, " ", "Separator1 ");
   string nameSep2 = StringConcatenate( UniqueNum, " ", tmpWindow, " ", "Separator2 ");
   
   string nBOX = StringConcatenate( "Static BoxSize - ", DoubleToStr(BoxSize, 0) );
   string nAVG = StringConcatenate( "AVG(",BarsToCalculateADR,")/",ADR_divisor," BoxSize - ", DoubleToStr(AVGBoxSize, 0) );
   string nSel = CharToStr(231);
   string nMeth1 = "Price - HIGH/LOW";
   string nMeth2 = "Price - CLOSE/CLOSE";
   string nSep1 = "- - - - - - - - - - - - - - -";
   
   ObjectCreate(nameBOX,OBJ_LABEL,tmpWindow,0,0,0,0);
   ObjectSet(nameBOX,OBJPROP_CORNER,1);
   ObjectSet(nameBOX,OBJPROP_XDISTANCE,tmpX);
   ObjectSet(nameBOX,OBJPROP_YDISTANCE,startY*2 + 2*FontSize + 5);
   ObjectSetText(nameBOX,nBOX,FontSize,"Arial",Yellow);
   
   ObjectCreate(nameAVG,OBJ_LABEL,tmpWindow,0,0,0,0);
   ObjectSet(nameAVG,OBJPROP_CORNER,1);
   ObjectSet(nameAVG,OBJPROP_XDISTANCE,tmpX);
   ObjectSet(nameAVG,OBJPROP_YDISTANCE,startY*2 + 4*FontSize + 5);
   ObjectSetText(nameAVG,nAVG,FontSize,"Arial",Yellow);
   
   ObjectDelete(nameSel);
   ObjectCreate(nameSel,OBJ_LABEL,tmpWindow,0,0,0,0);
   ObjectSet(nameSel,OBJPROP_CORNER,1);
   ObjectSet(nameSel,OBJPROP_XDISTANCE,0 + 1.1*FontSize);
   ObjectSet(nameSel,OBJPROP_YDISTANCE,tmpY);
   ObjectSetText(nameSel,nSel,FontSize,"Wingdings",Red);
   
   ObjectCreate(nameSep1,OBJ_LABEL,tmpWindow,0,0,0,0);
   ObjectSet(nameSep1,OBJPROP_CORNER,1);
   ObjectSet(nameSep1,OBJPROP_XDISTANCE,startX + 2*FontSize);
   ObjectSet(nameSep1,OBJPROP_YDISTANCE,startY*2 + 6*FontSize + 5);
   ObjectSetText(nameSep1,nSep1,FontSize,"Arial",Yellow);
   
   ObjectCreate(nameMeth,OBJ_LABEL,tmpWindow,0,0,0,0);
   ObjectSet(nameMeth,OBJPROP_CORNER,1);
   ObjectSet(nameMeth,OBJPROP_XDISTANCE,tmpX);
   ObjectSet(nameMeth,OBJPROP_YDISTANCE,startY*2 + 8*FontSize + 5);
   if (HL_CC == 1)
      ObjectSetText(nameMeth,nMeth1,FontSize,"Arial",Yellow);
   else
      ObjectSetText(nameMeth,nMeth2,FontSize,"Arial",Yellow);
   
   ObjectCreate(nameSep2,OBJ_LABEL,tmpWindow,0,0,0,0);
   ObjectSet(nameSep2,OBJPROP_CORNER,1);
   ObjectSet(nameSep2,OBJPROP_XDISTANCE,startX + 2*FontSize);
   ObjectSet(nameSep2,OBJPROP_YDISTANCE,startY*2 + 10*FontSize + 5);
   ObjectSetText(nameSep2,nSep1,FontSize,"Arial",Yellow);
   
   
}
//+------------------------------------------------------------------+ 
double CalculateAVG(int tBarsToCount)
{
   int   tempSum = 0;
   int   v;
   
   if (tBarsToCount<=0) tBarsToCount=1;
   
   for (v=1; v<=tBarsToCount; v++)
      tempSum = tempSum + getRange(v);
   
   tempSum = tempSum/tBarsToCount;

   return (NormalizeDouble(tempSum,0));
}
//+------------------------------------------------------------------+ 
double getRange(int barIdx)
{
   double   range;

      range = High[barIdx] - Low[barIdx];
      return (NormalizeDouble(range/Point,0));
}
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+

